# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: sikuli/v1/sikuli.proto
# Protobuf Python Version: 5.29.3
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    29,
    3,
    '',
    'sikuli/v1/sikuli.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x16sikuli/v1/sikuli.proto\x12\tsikuli.v1\"E\n\tGrayImage\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\r\n\x05width\x18\x02 \x01(\x05\x12\x0e\n\x06height\x18\x03 \x01(\x05\x12\x0b\n\x03pix\x18\x04 \x01(\x0c\"\x1d\n\x05Point\x12\t\n\x01x\x18\x01 \x01(\x05\x12\t\n\x01y\x18\x02 \x01(\x05\"2\n\x04Rect\x12\t\n\x01x\x18\x01 \x01(\x05\x12\t\n\x01y\x18\x02 \x01(\x05\x12\t\n\x01w\x18\x03 \x01(\x05\x12\t\n\x01h\x18\x04 \x01(\x05\"\xef\x01\n\x07Pattern\x12#\n\x05image\x18\x01 \x01(\x0b\x32\x14.sikuli.v1.GrayImage\x12\x17\n\nsimilarity\x18\x02 \x01(\x01H\x00\x88\x01\x01\x12\x12\n\x05\x65xact\x18\x03 \x01(\x08H\x01\x88\x01\x01\x12\'\n\rtarget_offset\x18\x04 \x01(\x0b\x32\x10.sikuli.v1.Point\x12\x1a\n\rresize_factor\x18\x05 \x01(\x01H\x02\x88\x01\x01\x12\"\n\x04mask\x18\x06 \x01(\x0b\x32\x14.sikuli.v1.GrayImageB\r\n\x0b_similarityB\x08\n\x06_exactB\x10\n\x0e_resize_factor\"f\n\x05Match\x12\x1d\n\x04rect\x18\x01 \x01(\x0b\x32\x0f.sikuli.v1.Rect\x12\r\n\x05score\x18\x02 \x01(\x01\x12 \n\x06target\x18\x03 \x01(\x0b\x32\x10.sikuli.v1.Point\x12\r\n\x05index\x18\x04 \x01(\x05\"[\n\tTextMatch\x12\x1d\n\x04rect\x18\x01 \x01(\x0b\x32\x0f.sikuli.v1.Rect\x12\x0c\n\x04text\x18\x02 \x01(\t\x12\x12\n\nconfidence\x18\x03 \x01(\x01\x12\r\n\x05index\x18\x04 \x01(\x05\"\xc9\x01\n\tOCRParams\x12\x10\n\x08language\x18\x01 \x01(\t\x12\x1a\n\x12training_data_path\x18\x02 \x01(\t\x12\x1b\n\x0emin_confidence\x18\x03 \x01(\x01H\x00\x88\x01\x01\x12\x1b\n\x0etimeout_millis\x18\x04 \x01(\x03H\x01\x88\x01\x01\x12\x1b\n\x0e\x63\x61se_sensitive\x18\x05 \x01(\x08H\x02\x88\x01\x01\x42\x11\n\x0f_min_confidenceB\x11\n\x0f_timeout_millisB\x11\n\x0f_case_sensitive\"J\n\x0cInputOptions\x12\x19\n\x0c\x64\x65lay_millis\x18\x01 \x01(\x03H\x00\x88\x01\x01\x12\x0e\n\x06\x62utton\x18\x02 \x01(\tB\x0f\n\r_delay_millis\"r\n\x0eObserveOptions\x12\x1c\n\x0finterval_millis\x18\x01 \x01(\x03H\x00\x88\x01\x01\x12\x1b\n\x0etimeout_millis\x18\x02 \x01(\x03H\x01\x88\x01\x01\x42\x12\n\x10_interval_millisB\x11\n\x0f_timeout_millis\"<\n\nAppOptions\x12\x1b\n\x0etimeout_millis\x18\x01 \x01(\x03H\x00\x88\x01\x01\x42\x11\n\x0f_timeout_millis\"I\n\x06Window\x12\r\n\x05title\x18\x01 \x01(\t\x12\x1f\n\x06\x62ounds\x18\x02 \x01(\x0b\x32\x0f.sikuli.v1.Rect\x12\x0f\n\x07\x66ocused\x18\x03 \x01(\x08\"\\\n\x0cObserveEvent\x12\x0c\n\x04type\x18\x01 \x01(\t\x12\x1f\n\x05match\x18\x02 \x01(\x0b\x32\x10.sikuli.v1.Match\x12\x1d\n\x15timestamp_unix_millis\x18\x03 \x01(\x03\"\x8a\x01\n\x0b\x46indRequest\x12$\n\x06source\x18\x01 \x01(\x0b\x32\x14.sikuli.v1.GrayImage\x12#\n\x07pattern\x18\x02 \x01(\x0b\x32\x12.sikuli.v1.Pattern\x12\x30\n\x0ematcher_engine\x18\x03 \x01(\x0e\x32\x18.sikuli.v1.MatcherEngine\"/\n\x0c\x46indResponse\x12\x1f\n\x05match\x18\x01 \x01(\x0b\x32\x10.sikuli.v1.Match\"4\n\x0f\x46indAllResponse\x12!\n\x07matches\x18\x01 \x03(\x0b\x32\x10.sikuli.v1.Match\"\xc9\x01\n\x12ScreenQueryOptions\x12\x1f\n\x06region\x18\x01 \x01(\x0b\x32\x0f.sikuli.v1.Rect\x12\x1b\n\x0etimeout_millis\x18\x02 \x01(\x03H\x00\x88\x01\x01\x12\x1c\n\x0finterval_millis\x18\x03 \x01(\x03H\x01\x88\x01\x01\x12\x30\n\x0ematcher_engine\x18\x04 \x01(\x0e\x32\x18.sikuli.v1.MatcherEngineB\x11\n\x0f_timeout_millisB\x12\n\x10_interval_millis\"g\n\x13\x46indOnScreenRequest\x12#\n\x07pattern\x18\x01 \x01(\x0b\x32\x12.sikuli.v1.Pattern\x12+\n\x04opts\x18\x02 \x01(\x0b\x32\x1d.sikuli.v1.ScreenQueryOptions\"i\n\x15\x45xistsOnScreenRequest\x12#\n\x07pattern\x18\x01 \x01(\x0b\x32\x12.sikuli.v1.Pattern\x12+\n\x04opts\x18\x02 \x01(\x0b\x32\x1d.sikuli.v1.ScreenQueryOptions\"I\n\x16\x45xistsOnScreenResponse\x12\x0e\n\x06\x65xists\x18\x01 \x01(\x08\x12\x1f\n\x05match\x18\x02 \x01(\x0b\x32\x10.sikuli.v1.Match\"g\n\x13WaitOnScreenRequest\x12#\n\x07pattern\x18\x01 \x01(\x0b\x32\x12.sikuli.v1.Pattern\x12+\n\x04opts\x18\x02 \x01(\x0b\x32\x1d.sikuli.v1.ScreenQueryOptions\"\x95\x01\n\x14\x43lickOnScreenRequest\x12#\n\x07pattern\x18\x01 \x01(\x0b\x32\x12.sikuli.v1.Pattern\x12+\n\x04opts\x18\x02 \x01(\x0b\x32\x1d.sikuli.v1.ScreenQueryOptions\x12+\n\nclick_opts\x18\x03 \x01(\x0b\x32\x17.sikuli.v1.InputOptions\"]\n\x0fReadTextRequest\x12$\n\x06source\x18\x01 \x01(\x0b\x32\x14.sikuli.v1.GrayImage\x12$\n\x06params\x18\x02 \x01(\x0b\x32\x14.sikuli.v1.OCRParams\" \n\x10ReadTextResponse\x12\x0c\n\x04text\x18\x01 \x01(\t\"l\n\x0f\x46indTextRequest\x12$\n\x06source\x18\x01 \x01(\x0b\x32\x14.sikuli.v1.GrayImage\x12\r\n\x05query\x18\x02 \x01(\t\x12$\n\x06params\x18\x03 \x01(\x0b\x32\x14.sikuli.v1.OCRParams\"9\n\x10\x46indTextResponse\x12%\n\x07matches\x18\x01 \x03(\x0b\x32\x14.sikuli.v1.TextMatch\"O\n\x10MoveMouseRequest\x12\t\n\x01x\x18\x01 \x01(\x05\x12\t\n\x01y\x18\x02 \x01(\x05\x12%\n\x04opts\x18\x03 \x01(\x0b\x32\x17.sikuli.v1.InputOptions\"K\n\x0c\x43lickRequest\x12\t\n\x01x\x18\x01 \x01(\x05\x12\t\n\x01y\x18\x02 \x01(\x05\x12%\n\x04opts\x18\x03 \x01(\x0b\x32\x17.sikuli.v1.InputOptions\"F\n\x0fTypeTextRequest\x12\x0c\n\x04text\x18\x01 \x01(\t\x12%\n\x04opts\x18\x02 \x01(\x0b\x32\x17.sikuli.v1.InputOptions\"\x1d\n\rHotkeyRequest\x12\x0c\n\x04keys\x18\x01 \x03(\t\"\x10\n\x0e\x41\x63tionResponse\"\xa5\x01\n\x0eObserveRequest\x12$\n\x06source\x18\x01 \x01(\x0b\x32\x14.sikuli.v1.GrayImage\x12\x1f\n\x06region\x18\x02 \x01(\x0b\x32\x0f.sikuli.v1.Rect\x12#\n\x07pattern\x18\x03 \x01(\x0b\x32\x12.sikuli.v1.Pattern\x12\'\n\x04opts\x18\x04 \x01(\x0b\x32\x19.sikuli.v1.ObserveOptions\"\x86\x01\n\x14ObserveChangeRequest\x12$\n\x06source\x18\x01 \x01(\x0b\x32\x14.sikuli.v1.GrayImage\x12\x1f\n\x06region\x18\x02 \x01(\x0b\x32\x0f.sikuli.v1.Rect\x12\'\n\x04opts\x18\x03 \x01(\x0b\x32\x19.sikuli.v1.ObserveOptions\":\n\x0fObserveResponse\x12\'\n\x06\x65vents\x18\x01 \x03(\x0b\x32\x17.sikuli.v1.ObserveEvent\"S\n\x10\x41ppActionRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0c\n\x04\x61rgs\x18\x02 \x03(\t\x12#\n\x04opts\x18\x03 \x01(\x0b\x32\x15.sikuli.v1.AppOptions\"\'\n\x14IsAppRunningResponse\x12\x0f\n\x07running\x18\x01 \x01(\x08\"9\n\x13ListWindowsResponse\x12\"\n\x07windows\x18\x01 \x03(\x0b\x32\x11.sikuli.v1.Window*\xe5\x01\n\rMatcherEngine\x12\x1e\n\x1aMATCHER_ENGINE_UNSPECIFIED\x10\x00\x12\x1b\n\x17MATCHER_ENGINE_TEMPLATE\x10\x01\x12\x16\n\x12MATCHER_ENGINE_ORB\x10\x02\x12\x19\n\x15MATCHER_ENGINE_HYBRID\x10\x03\x12\x18\n\x14MATCHER_ENGINE_AKAZE\x10\x04\x12\x18\n\x14MATCHER_ENGINE_BRISK\x10\x05\x12\x17\n\x13MATCHER_ENGINE_KAZE\x10\x06\x12\x17\n\x13MATCHER_ENGINE_SIFT\x10\x07\x32\x8c\x0b\n\rSikuliService\x12\x37\n\x04\x46ind\x12\x16.sikuli.v1.FindRequest\x1a\x17.sikuli.v1.FindResponse\x12=\n\x07\x46indAll\x12\x16.sikuli.v1.FindRequest\x1a\x1a.sikuli.v1.FindAllResponse\x12G\n\x0c\x46indOnScreen\x12\x1e.sikuli.v1.FindOnScreenRequest\x1a\x17.sikuli.v1.FindResponse\x12U\n\x0e\x45xistsOnScreen\x12 .sikuli.v1.ExistsOnScreenRequest\x1a!.sikuli.v1.ExistsOnScreenResponse\x12G\n\x0cWaitOnScreen\x12\x1e.sikuli.v1.WaitOnScreenRequest\x1a\x17.sikuli.v1.FindResponse\x12I\n\rClickOnScreen\x12\x1f.sikuli.v1.ClickOnScreenRequest\x1a\x17.sikuli.v1.FindResponse\x12\x43\n\x08ReadText\x12\x1a.sikuli.v1.ReadTextRequest\x1a\x1b.sikuli.v1.ReadTextResponse\x12\x43\n\x08\x46indText\x12\x1a.sikuli.v1.FindTextRequest\x1a\x1b.sikuli.v1.FindTextResponse\x12\x43\n\tMoveMouse\x12\x1b.sikuli.v1.MoveMouseRequest\x1a\x19.sikuli.v1.ActionResponse\x12;\n\x05\x43lick\x12\x17.sikuli.v1.ClickRequest\x1a\x19.sikuli.v1.ActionResponse\x12\x41\n\x08TypeText\x12\x1a.sikuli.v1.TypeTextRequest\x1a\x19.sikuli.v1.ActionResponse\x12=\n\x06Hotkey\x12\x18.sikuli.v1.HotkeyRequest\x1a\x19.sikuli.v1.ActionResponse\x12\x46\n\rObserveAppear\x12\x19.sikuli.v1.ObserveRequest\x1a\x1a.sikuli.v1.ObserveResponse\x12\x46\n\rObserveVanish\x12\x19.sikuli.v1.ObserveRequest\x1a\x1a.sikuli.v1.ObserveResponse\x12L\n\rObserveChange\x12\x1f.sikuli.v1.ObserveChangeRequest\x1a\x1a.sikuli.v1.ObserveResponse\x12\x41\n\x07OpenApp\x12\x1b.sikuli.v1.AppActionRequest\x1a\x19.sikuli.v1.ActionResponse\x12\x42\n\x08\x46ocusApp\x12\x1b.sikuli.v1.AppActionRequest\x1a\x19.sikuli.v1.ActionResponse\x12\x42\n\x08\x43loseApp\x12\x1b.sikuli.v1.AppActionRequest\x1a\x19.sikuli.v1.ActionResponse\x12L\n\x0cIsAppRunning\x12\x1b.sikuli.v1.AppActionRequest\x1a\x1f.sikuli.v1.IsAppRunningResponse\x12J\n\x0bListWindows\x12\x1b.sikuli.v1.AppActionRequest\x1a\x1e.sikuli.v1.ListWindowsResponseB8Z6github.com/smysnk/sikuligo/internal/grpcv1/pb;sikuliv1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'sikuli.v1.sikuli_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'Z6github.com/smysnk/sikuligo/internal/grpcv1/pb;sikuliv1'
  _globals['_MATCHERENGINE']._serialized_start=3377
  _globals['_MATCHERENGINE']._serialized_end=3606
  _globals['_GRAYIMAGE']._serialized_start=37
  _globals['_GRAYIMAGE']._serialized_end=106
  _globals['_POINT']._serialized_start=108
  _globals['_POINT']._serialized_end=137
  _globals['_RECT']._serialized_start=139
  _globals['_RECT']._serialized_end=189
  _globals['_PATTERN']._serialized_start=192
  _globals['_PATTERN']._serialized_end=431
  _globals['_MATCH']._serialized_start=433
  _globals['_MATCH']._serialized_end=535
  _globals['_TEXTMATCH']._serialized_start=537
  _globals['_TEXTMATCH']._serialized_end=628
  _globals['_OCRPARAMS']._serialized_start=631
  _globals['_OCRPARAMS']._serialized_end=832
  _globals['_INPUTOPTIONS']._serialized_start=834
  _globals['_INPUTOPTIONS']._serialized_end=908
  _globals['_OBSERVEOPTIONS']._serialized_start=910
  _globals['_OBSERVEOPTIONS']._serialized_end=1024
  _globals['_APPOPTIONS']._serialized_start=1026
  _globals['_APPOPTIONS']._serialized_end=1086
  _globals['_WINDOW']._serialized_start=1088
  _globals['_WINDOW']._serialized_end=1161
  _globals['_OBSERVEEVENT']._serialized_start=1163
  _globals['_OBSERVEEVENT']._serialized_end=1255
  _globals['_FINDREQUEST']._serialized_start=1258
  _globals['_FINDREQUEST']._serialized_end=1396
  _globals['_FINDRESPONSE']._serialized_start=1398
  _globals['_FINDRESPONSE']._serialized_end=1445
  _globals['_FINDALLRESPONSE']._serialized_start=1447
  _globals['_FINDALLRESPONSE']._serialized_end=1499
  _globals['_SCREENQUERYOPTIONS']._serialized_start=1502
  _globals['_SCREENQUERYOPTIONS']._serialized_end=1703
  _globals['_FINDONSCREENREQUEST']._serialized_start=1705
  _globals['_FINDONSCREENREQUEST']._serialized_end=1808
  _globals['_EXISTSONSCREENREQUEST']._serialized_start=1810
  _globals['_EXISTSONSCREENREQUEST']._serialized_end=1915
  _globals['_EXISTSONSCREENRESPONSE']._serialized_start=1917
  _globals['_EXISTSONSCREENRESPONSE']._serialized_end=1990
  _globals['_WAITONSCREENREQUEST']._serialized_start=1992
  _globals['_WAITONSCREENREQUEST']._serialized_end=2095
  _globals['_CLICKONSCREENREQUEST']._serialized_start=2098
  _globals['_CLICKONSCREENREQUEST']._serialized_end=2247
  _globals['_READTEXTREQUEST']._serialized_start=2249
  _globals['_READTEXTREQUEST']._serialized_end=2342
  _globals['_READTEXTRESPONSE']._serialized_start=2344
  _globals['_READTEXTRESPONSE']._serialized_end=2376
  _globals['_FINDTEXTREQUEST']._serialized_start=2378
  _globals['_FINDTEXTREQUEST']._serialized_end=2486
  _globals['_FINDTEXTRESPONSE']._serialized_start=2488
  _globals['_FINDTEXTRESPONSE']._serialized_end=2545
  _globals['_MOVEMOUSEREQUEST']._serialized_start=2547
  _globals['_MOVEMOUSEREQUEST']._serialized_end=2626
  _globals['_CLICKREQUEST']._serialized_start=2628
  _globals['_CLICKREQUEST']._serialized_end=2703
  _globals['_TYPETEXTREQUEST']._serialized_start=2705
  _globals['_TYPETEXTREQUEST']._serialized_end=2775
  _globals['_HOTKEYREQUEST']._serialized_start=2777
  _globals['_HOTKEYREQUEST']._serialized_end=2806
  _globals['_ACTIONRESPONSE']._serialized_start=2808
  _globals['_ACTIONRESPONSE']._serialized_end=2824
  _globals['_OBSERVEREQUEST']._serialized_start=2827
  _globals['_OBSERVEREQUEST']._serialized_end=2992
  _globals['_OBSERVECHANGEREQUEST']._serialized_start=2995
  _globals['_OBSERVECHANGEREQUEST']._serialized_end=3129
  _globals['_OBSERVERESPONSE']._serialized_start=3131
  _globals['_OBSERVERESPONSE']._serialized_end=3189
  _globals['_APPACTIONREQUEST']._serialized_start=3191
  _globals['_APPACTIONREQUEST']._serialized_end=3274
  _globals['_ISAPPRUNNINGRESPONSE']._serialized_start=3276
  _globals['_ISAPPRUNNINGRESPONSE']._serialized_end=3315
  _globals['_LISTWINDOWSRESPONSE']._serialized_start=3317
  _globals['_LISTWINDOWSRESPONSE']._serialized_end=3374
  _globals['_SIKULISERVICE']._serialized_start=3609
  _globals['_SIKULISERVICE']._serialized_end=5029
# @@protoc_insertion_point(module_scope)
